/*----------------------------------------------------------------------------
 *      PSTU MDR32F9Q2I projects
 *----------------------------------------------------------------------------
 *      Name:    GLCD_MT12864.c
 *      Purpose: MT12864 graphic LCD driver
 *      Rev.:    V1.00
 *----------------------------------------------------------------------------
 *      Author:	 			A. Sharonov
 *      Created:			29.09.2015
 *			Last modifed:	21.10.2015
 *---------------------------------------------------------------------------*/
 
#include <MDR32F9x.h>
#include <MDR32F9x_config.h>
#include "GLCD.h"
#include "Font_12x16.h"
#include "Font_6x8.h"
#include "Font_7x10_thin.h"
#include "Font_7x10_bold.h"

/* LCD pins

A0	pin PC0
E		pin PC1
E1	pin PB7
E2	pin PB8
RES	pin PB9
RW	pin PB10

D0-D5	- PA0-PA5
D6-D7	- PF2-PF3
*/

#define A0				0x0001
#define E					0x0002
#define E1				0x0080
#define E2				0x0100
#define RES				0x0200
#define RW				0x0400

#define LCD_D0D5	0x003F
#define LCD_D6D7	0x000C

//  
#define MAX_X 127
#define MAX_Y 63 

#define BG_COLOR  0                     /* Background color                   */
#define TXT_COLOR 1                     /* Text color                         */

static volatile unsigned short Color[2] = {White, Black};

static void delay (unsigned int cnt)
{
  while (cnt--);
}

static unsigned char LCD_read_status (void)
{
	unsigned char status;

	MDR_PORTA->RXTX				&=~LCD_D0D5;			//Clear D0..D5 pins
	MDR_PORTF->RXTX				&=~LCD_D6D7;			//Clear D6, D7 pins
	MDR_PORTA->OE					&=~LCD_D0D5;			//D0..D5 to input
	MDR_PORTF->OE					&=~LCD_D6D7;			//D6, D7 to input
	MDR_PORTC->RXTX 			&= ~A0;						//Clear A0 pin (cmd mode enable)
	MDR_PORTB->RXTX				|= RW;						//Set RW pin (read mode)
	delay(50);
	MDR_PORTC->RXTX				|= E;							//Set E pin
	delay(50);
	status=0;
	status=(MDR_PORTA->RXTX &0x003F);				//Read D0..D5 from LCD
	status|=(MDR_PORTF->RXTX &0x000C)<<4;		//Read D6, D7 from LCD
	MDR_PORTC->RXTX				&= ~E;						//Clear E pin
	return (status);
}

static void LCD_wait_busy (void)
{
	unsigned char status=0;
	
	do
		status= LCD_read_status();
	while (status &0x80);
}

static unsigned char LCD_read_data (void)
{
	unsigned char data;

	LCD_wait_busy();
	MDR_PORTA->RXTX				&=~LCD_D0D5;			//Clear D0..D5 pins
	MDR_PORTF->RXTX				&=~LCD_D6D7;			//Clear D6, D7 pins
	MDR_PORTF->OE					&=~LCD_D0D5;			//D0..D5 to input
	MDR_PORTA->OE					&=~LCD_D6D7;			//D6, D7 to input
	MDR_PORTC->RXTX 			|= A0;						//Set A0 pin (cmd mode enable)
	MDR_PORTB->RXTX				|= RW;						//Set RW pin (read mode)
	delay(50);
	MDR_PORTC->RXTX				|= E;							//Set E pin
	delay(50);
	data=0;
	data=(MDR_PORTA->RXTX &0x003F);					//Read D0..D5 from LCD
	data |=(MDR_PORTF->RXTX &0x000C)<<4;		//Read D6, D7 from LCD
	MDR_PORTC->RXTX				&= ~E;						//Clear E pin
	return (data);
}

static void LCD_write (unsigned char data)
{
	MDR_PORTA->RXTX				&=~LCD_D0D5;			//Clear D0..D5 pins
	MDR_PORTF->RXTX				&=~LCD_D6D7;			//Clear D6, D7 pins
	MDR_PORTA->OE					|=LCD_D0D5;				//D0..D5 to output
	MDR_PORTF->OE					|=LCD_D6D7;				//D6, D7 to output
	MDR_PORTB->RXTX				&= ~RW;						//LCD to write mode
	//MDR_PORTC->RXTX 			&= ~A0;						//Clear A0 pin (cmd mode enable)
	delay(50);
	MDR_PORTC->RXTX				|= E;							//Set E pin
	MDR_PORTA->RXTX				|=(data &0x003F);	//Write D0..D5 to LCD
	MDR_PORTF->RXTX		|=(data >>4)&0x000C;	//Write D6, D7 to LCD
	delay(50);
	MDR_PORTC->RXTX				&= ~E;						//Clear E pin
}

static void LCD_write_cmd (unsigned char cmd)
{
	LCD_wait_busy();
	MDR_PORTC->RXTX &= ~A0;									//Clear A0 pin (cmd mode enable)
	LCD_write(cmd);
}

static void LCD_write_data (unsigned char data)
{
	LCD_wait_busy();
	MDR_PORTC->RXTX |= A0;									//Set A0 pin (data mode enable)
	LCD_write(data);
}

/************************ Exported functions **********************************/

/*******************************************************************************
* Initialize the Graphic LCD controller                                        *
*   Parameter:                                                                 *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Init(void)
{
	//PORTA init
	MDR_RST_CLK->PER_CLOCK	|=(1<<21);													//Enable PORTA in PER_CLOCK
	MDR_PORTA->OE						|=0x003F;														//PA0..PA5 is output
	MDR_PORTA->ANALOG				|=0x003F;														//PA0..PA5 is digital
	MDR_PORTA->PWR					|=0x00000FFF;												//PA0..PA5 is fast front
	//PORTB init
	MDR_RST_CLK->PER_CLOCK	|=(1<<22);													//Enable PORTB in PER_CLOCK
	MDR_PORTB->OE						|=0x0780;														//PB7..PB10 is output
	MDR_PORTB->ANALOG				|=0x0780;														//PB7..PB10 is digital
	MDR_PORTB->PWR					|=0x003FC000;												//PB7..PB10 is fast front
	//PORTC init
	MDR_RST_CLK->PER_CLOCK	|=(1<<23);													//Enable PORTC in PER_CLOCK
	MDR_PORTC->OE						|=0x0003;														//PC0, PC1 is output
	MDR_PORTC->ANALOG				|=0x0003;														//PC0, PC1 is digital
	MDR_PORTC->PWR 					|=0x000F;														//PC0, PC1 is fast front
	//PORTF init
	MDR_RST_CLK->PER_CLOCK	|=(1<<29);													//Enable PORTF in PER_CLOCK
	MDR_PORTF->OE						|=0x000C;														//PF2, PF3 is output
	MDR_PORTF->ANALOG				|=0x000C;														//PF2, PF3 is digital
	MDR_PORTF->PWR					|=0x00F0;														//PF2, PF3 is fast front
	//Reset all control lines to 0
	MDR_PORTB->RXTX					&=~0x0780;													//Reset PB7..PB10 (include RES) to 0
	MDR_PORTC->RXTX					&=~0x0003;													//Reset PC0, PC1 (A0, E) to 0
	delay(0xFFFF);																							//Wait 100ms
	MDR_PORTB->RXTX					|=RES;															//Restore RES pin to 1
	MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
	LCD_write_cmd(0x3F);																				//LCD on
	LCD_write_cmd(0x40);
	LCD_write_cmd(0xC0);																				//Set start line
	MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
	MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
	LCD_wait_busy();
	LCD_write_cmd(0x3F);																				//LCD on
	LCD_write_cmd(0x40);
	LCD_write_cmd(0xC0);																				//Set start line
}

/*******************************************************************************
* Clear display                                                                *
*   Parameter:      color:    display clearing color                           *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Clear (unsigned short color)
{
	unsigned char i=0, j=0;
	
	MDR_PORTB->RXTX					&=~E2;															//Unselect second chip in LCD
	delay(500);
	MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
	//LCD_write_cmd(0x40);
	LCD_write_cmd(0x3E);																				//LCD off
	LCD_write_cmd(0xC0);																				//Set start line
	
	for (i=0; i<8; i++)
	{
		LCD_write_cmd(0xB8 | i);
		for (j=0; j<64; j++)
		{
			if (color==White)
				LCD_write_data(0x00);
			else
				LCD_write_data(0xFF);
		}
	}
	LCD_write_cmd(0x3F);																				//LCD on
	MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
	delay(500);
	MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
	LCD_write_cmd(0x3E);																				//LCD off
	LCD_write_cmd(0xC0);																				//Set start line
	for (i=0; i<8; i++)
	{
		LCD_write_cmd(0xB8 | i);
		for (j=0; j<64; j++)
		{
			if (color==White)
				LCD_write_data(0x00);
			else
				LCD_write_data(0xFF);
		}
	}
	LCD_write_cmd(0x3F);																				//LCD on
}

/*******************************************************************************
* Draw a pixel in foreground color                                             *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_PutPixel (unsigned int x, unsigned int y) {

  unsigned char bite;
  unsigned char page; 
  unsigned char data;
  
  //      
  if((x>MAX_X)|(y>MAX_Y)) return;
  // 
  if(x>=64)
	{
    MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
    MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
    x=x-64;
  }
  else
	{
    MDR_PORTB->RXTX					&=~E2;															//Unselect second chip in LCD
    MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
  }   
  //page-  
  page=y/8;
  //bite-     
  bite=y%8;
  //   
  LCD_write_cmd(0xB8 | page);																		//Set page
	LCD_write_cmd(0x40 | x);																			//Set address
  //   LCD(2  ..   )
  data=LCD_read_data(); 
  data=LCD_read_data(); 
  //       .
  //    ,      
	if (Color[TXT_COLOR]==White)
	{
		data &=~(1<<bite);
	}
	else
	{
		data=data|(1<<bite);
	}
  LCD_write_cmd(0x40 | x);																			//Set address
  LCD_write_data(data);
}

/*******************************************************************************
* Set foreground color                                                         *
*   Parameter:      color:    foreground color                                 *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_SetTextColor (unsigned short color) {

  Color[TXT_COLOR] = color;
}

/*******************************************************************************
* Set background color                                                         *
*   Parameter:      color:    background color                                 *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_SetBackColor (unsigned short color) {

  Color[BG_COLOR] = color;
}

/*******************************************************************************
* Draw character on given position                                             *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   cw:       character width in pixel                         *
*                   ch:       character height in pixels                       *
*                   c:        pointer to character bitmap                      *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_DrawChar (unsigned int x, unsigned int y, unsigned int cw, unsigned int ch, unsigned char *c) 
{
	unsigned char page;
	unsigned char byte;
	unsigned char tmp_data;
	unsigned char t=0;
	unsigned char mode=0;
	//unsigned char temp=0;
	
  if((x>MAX_X)|(y>MAX_Y)) return;
	if (!(Color[BG_COLOR]==White))
	{
		mode |=0x01;
	}
	if (!(Color[TXT_COLOR]==White))
	{
		mode |=0x02;
	}
  //k  = (cw + 7)/8;
	if (cw<7)
	{
		//      
		page=y/8;
		byte=y%8;		
		for (t=0; t<6; t++)
		{
			//   63   
			if((x+t)>=64)
			{
				MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
				MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
				//x=x-64;
			}
			else
			{
				MDR_PORTB->RXTX					&=~E2;															//Unselect second chip in LCD
				MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
			}				
			//   (     )
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																			//Set address
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();  
			//  
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																	//Set address
			switch (mode)
			{
				case 0: LCD_write_data((0x00<<byte)|tmp_data); 		break;
				case 1: LCD_write_data((((~c[t])&0xFF)<<byte)|tmp_data); break;
				case 2: LCD_write_data((c[t]<<byte)|tmp_data); 		break;
				case 3: LCD_write_data((0xFF<<byte)|tmp_data); 		break;
			}
			if (byte)
			{
				//   63   
				if((x+t)>=64)
				{
					MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
					MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
				//x=x-64;
				}
				else
				{
					MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
					MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
				}				
				//   (     )
				LCD_write_cmd(0xB8 | page+1);																		//Set page
				LCD_write_cmd(0x40 | (x+t));																			//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page+1);																	//Set page
				LCD_write_cmd(0x40 | (x+t));																//Set address
				LCD_write_data((c[t]>>(8-byte))|(tmp_data));				
				switch (mode)
				{
					case 0: LCD_write_data((0x00>>(8-byte))|(tmp_data)); 		break;
					case 1: LCD_write_data((((~c[t])&0xFF)>>(8-byte))|(tmp_data)); break;
					case 2: LCD_write_data((c[t]>>(8-byte))|(tmp_data)); 		break;
					case 3: LCD_write_data((0xFF>>(8-byte))|(tmp_data)); 		break;
				}
			}
		}
	}
	if ((cw>6)&&(cw<12))
	{
		//      
		page=y/8;
		byte=y%8;		
		for (t=0; t<7; t++)
		{
			//   63   
			if((x+t)>=64)
			{
				MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
				MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
				//x=x-64;
			}
			else
			{
				MDR_PORTB->RXTX					&=~E2;															//Unselect second chip in LCD
				MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
			}				
			//   (     )
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																	//Set address
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();  
			//  
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																	//Set address
			switch (mode)
			{
				case 0: LCD_write_data((0x00<<byte)|tmp_data); 						break;
				case 1: LCD_write_data((((~c[t])<<byte)&0xFF)|tmp_data); 	break;
				case 2: LCD_write_data((c[t]<<byte)|tmp_data); 						break;
				case 3: LCD_write_data((0xFF<<byte)|tmp_data); 						break;
			}
			if (byte)
			{
				//   63   
				if((x+t)>=64)
				{
					MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
					MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
					//x=x-64;
				}
				else
				{
					MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
					MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
				}				
				//   (     )
				LCD_write_cmd(0xB8 | page+1);																//Set page
				LCD_write_cmd(0x40 | (x+t));																//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page+1);																//Set page
				LCD_write_cmd(0x40 | (x+t));																//Set address
				//LCD_write_data((c[t]>>(8-byte))|(tmp_data));
				switch (mode)
				{
					case 0: LCD_write_data((0x00>>(8-byte))|(tmp_data)); 							break;
					case 1: LCD_write_data((((~c[t])&0xFF)>>(8-byte))|(tmp_data)); 		break;
					case 2: LCD_write_data((c[t]>>(8-byte))|(tmp_data)); 							break;
					case 3: LCD_write_data((0xFF>>(8-byte))|(tmp_data)); 							break;
				}
			}
		}		
		//      
		page=y/8+1;
		byte=y%8;		
		for (t=0; t<7; t++)
		{
			//   63   
			if((x+t)>=64)
			{
				MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
				MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
				//x=x-64;
			}
			else
			{
				MDR_PORTB->RXTX					&=~E2;															//Unselect second chip in LCD
				MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
			}				
			//   (     )
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																			//Set address
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();  
			//  
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																	//Set address
			//LCD_write_data((c[t+7]<<byte)|tmp_data);
			switch (mode)
			{
				case 0: LCD_write_data((0x00<<byte)|tmp_data); 		break;
				case 1: LCD_write_data((((~c[t+7])&0x03)<<byte)|tmp_data); break;
				case 2: LCD_write_data((c[t+7]<<byte)|tmp_data); 		break;
				case 3: LCD_write_data((0x03<<byte)|tmp_data); 		break;
			}
			if (byte)
			{
				//   63   
				if((x+t)>=64)
				{
					MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
					MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
				//x=x-64;
				}
				else
				{
					MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
					MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
				}				
				//   (     )
				LCD_write_cmd(0xB8 | page+1);																		//Set page
				LCD_write_cmd(0x40 | (x+t));																			//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page+1);																	//Set page
				LCD_write_cmd(0x40 | (x+t));																//Set address
				//LCD_write_data((c[t+7]>>(8-byte))|(tmp_data));
				switch (mode)
				{
					case 0: LCD_write_data((0x00>>(8-byte))|(tmp_data)); 		break;
					case 1: LCD_write_data((((~c[t+7])&0x03)>>(8-byte))|(tmp_data)); break;
					case 2: LCD_write_data((c[t+7]>>(8-byte))|(tmp_data)); 		break;
					case 3: LCD_write_data((0x03>>(8-byte))|(tmp_data)); 		break;
				}
			}
		}//*/		
	}
	if (cw>11)
	{
		//      
		page=y/8;
		byte=y%8;		
		for (t=0; t<12; t++)
		{
			//   63   
			if((x+t)>=64)
			{
				MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
				MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
				//x=x-64;
			}
			else
			{
				MDR_PORTB->RXTX					&=~E2;															//Unselect second chip in LCD
				MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
			}				
			//   (     )
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																	//Set address
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();  
			//  
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																	//Set address
			switch (mode)
			{
				case 0: LCD_write_data((0x00<<byte)|tmp_data); 		break;
				case 1: LCD_write_data((((~c[t])&0xFF)<<byte)|tmp_data); break;
				case 2: LCD_write_data((c[t]<<byte)|tmp_data); 		break;
				case 3: LCD_write_data((0xFF<<byte)|tmp_data); 		break;
			}
			if (byte)
			{
				//   63   
				if((x+t)>=64)
				{
					MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
					MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
					//x=x-64;
				}
				else
				{
					MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
					MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
				}				
				//   (     )
				LCD_write_cmd(0xB8 | page+1);																//Set page
				LCD_write_cmd(0x40 | (x+t));																//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page+1);																//Set page
				LCD_write_cmd(0x40 | (x+t));																//Set address
				//LCD_write_data((c[t]>>(8-byte))|(tmp_data));				
				switch (mode)
				{
					case 0: LCD_write_data((0x00>>(8-byte))|(tmp_data)); 						break;
					case 1: LCD_write_data((((~c[t])&0xFF)>>(8-byte))|(tmp_data)); 	break;
					case 2: LCD_write_data((c[t]>>(8-byte))|(tmp_data)); 						break;
					case 3: LCD_write_data((0xFF>>(8-byte))|(tmp_data)); 						break;
				}
			}
		}		
		//      
		page=y/8+1;
		byte=y%8;		
		for (t=0; t<12; t++)
		{
			//   63   
			if((x+t)>=64)
			{
				MDR_PORTB->RXTX					&=~E1;															//Unselect first chip in LCD
				MDR_PORTB->RXTX					|=E2;																//Select second chip in LCD
				//x=x-64;
			}
			else
			{
				MDR_PORTB->RXTX					&=~E2;															//Unselect second chip in LCD
				MDR_PORTB->RXTX					|=E1;																//Select first chip in LCD
			}				
			//   (     )
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																			//Set address
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();  
			//  
			LCD_write_cmd(0xB8 | page);																		//Set page
			LCD_write_cmd(0x40 | (x+t));																	//Set address
			//LCD_write_data((c[t+12]<<byte)|tmp_data);
			switch (mode)
			{
				case 0: LCD_write_data((0x00<<byte)|tmp_data); 								break;
				case 1: LCD_write_data((((~c[t+12])&0xFF)<<byte)|tmp_data); 	break;
				case 2: LCD_write_data((c[t+12]<<byte)|tmp_data); 						break;
				case 3: LCD_write_data((0xFF<<byte)|tmp_data); 								break;
			}
			if (byte)
			{
				//   63   
				if((x+t)>=64)
				{
					MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
					MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
				//x=x-64;
				}
				else
				{
					MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
					MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
				}				
				//   (     )
				LCD_write_cmd(0xB8 | page+1);																		//Set page
				LCD_write_cmd(0x40 | (x+t));																			//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page+1);																	//Set page
				LCD_write_cmd(0x40 | (x+t));																//Set address
				//LCD_write_data((c[t+12]>>(8-byte))|(tmp_data));				
				switch (mode)
				{
					case 0: LCD_write_data((0x00>>(8-byte))|(tmp_data)); 								break;
					case 1: LCD_write_data((((~c[t+12])&0xFF)>>(8-byte))|(tmp_data)); 	break;
					case 2: LCD_write_data((c[t+12]>>(8-byte))|(tmp_data)); 						break;
					case 3: LCD_write_data((0xFF>>(8-byte))|(tmp_data)); 								break;
				}
			}
		}//*/
	}
}


/************************************************************************************
* Disply character on given line                                               			*
*   Parameter:      ln:       line number                                      			*
*                   col:      column number                                    			*
*                   fi:       font index (0=6x8, 1=7x10 thin, 2=7x10 bold, 3=12x16) *
*                   c:        ascii character                                  			*
*   Return:                                                                    			*
************************************************************************************/

void GLCD_DisplayChar (unsigned int ln, unsigned int col, unsigned char fi, unsigned char c) {

  switch (fi) {
    case 0:  /* Font 6 x 8 */
      GLCD_DrawChar(col *  6, ln *  8,  6,  8, 	(unsigned char *)&Font_6x8_Data  [c * 6]);
      break;
    case 1:  /* Font 7 x 10 thin */
      GLCD_DrawChar(col * 7, ln * 10, 7, 10, 		(unsigned char *)&Font_7x10_thin_Data[c * 14]);
      break;
		case 2:  /* Font 7 x 10 bold */
      GLCD_DrawChar(col * 7, ln * 10, 7, 10, 		(unsigned char *)&Font_7x10_bold_Data[c * 14]);
      break;
		case 3:  /* Font 12 x 16 */
      GLCD_DrawChar(col * 12, ln * 16, 12, 16, 	(unsigned char *)&Font_12x16_Data[c * 24]);
      break;
  }
}


/************************************************************************************
* Disply string on given line                                                       *
*   Parameter:      ln:       line number                                           *
*                   col:      column number                                         *
*                   fi:				font index (0=6x8, 1=7x10 thin, 2=7x10 bold, 3=12x16) *
*                   s:        pointer to string                                     *
*   Return:                                                                         *
************************************************************************************/

void GLCD_DisplayString (unsigned int ln, unsigned int col, unsigned char fi, unsigned char *s) {

  while (*s) {
    GLCD_DisplayChar(ln, col++, fi, *s++);
  }
}

/************************************************************************************
* Clear given line                                                                  *
*   Parameter:      ln:       line number                                           *
*                   fi:				font index (0=6x8, 1=7x10 thin, 2=7x10 bold, 3=12x16) *
*   Return:                                                                         *
************************************************************************************/

void GLCD_ClearLn (unsigned int ln, unsigned char fi) 
{
  unsigned char i;
	unsigned char page=0;
	unsigned char byte=0;
	unsigned char tmp_data=0;
	
	switch (fi &0x03)
	{
		case 0:
		{
			page=ln;
			MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
			delay(500);
			MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
			LCD_write_cmd(0xC0);																				//Set start line
			LCD_write_cmd(0xB8 | page);
			for (i=0; i<64; i++)
			{
				if (Color[BG_COLOR]==White)
					LCD_write_data(0x00);
				else
					LCD_write_data(0xFF);				
			}
			MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
			delay(500);
			MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
			LCD_write_cmd(0xC0);																			//Set start line
			LCD_write_cmd(0xB8 | page);
			for (i=0; i<64; i++)
			{
				if (Color[BG_COLOR]==White)
					LCD_write_data(0x00);
				else
					LCD_write_data(0xFF);				
			}
			break;
		}
		case 3:
		{
			page=ln*2;
			MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
			delay(500);
			MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
			LCD_write_cmd(0xC0);																				//Set start line
			LCD_write_cmd(0xB8 | page);
			for (i=0; i<64; i++)
			{
				if (Color[BG_COLOR]==White)
					LCD_write_data(0x00);
				else
					LCD_write_data(0xFF);				
			}
			LCD_write_cmd(0xC0);																				//Set start line
			LCD_write_cmd(0xB8 | page+1);
			for (i=0; i<64; i++)
			{
				if (Color[BG_COLOR]==White)
					LCD_write_data(0x00);
				else
					LCD_write_data(0xFF);				
			}
			MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
			delay(500);
			MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
			LCD_write_cmd(0xC0);																			//Set start line
			LCD_write_cmd(0xB8 | page);
			for (i=0; i<64; i++)
			{
				if (Color[BG_COLOR]==White)
					LCD_write_data(0x00);
				else
					LCD_write_data(0xFF);				
			}
			LCD_write_cmd(0xC0);																			//Set start line
			LCD_write_cmd(0xB8 | page+1);
			for (i=0; i<64; i++)
			{
				if (Color[BG_COLOR]==White)
					LCD_write_data(0x00);
				else
					LCD_write_data(0xFF);				
			}
			break;
		}
		default:
		{
			MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
			delay(500);
			MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
			page=ln*10/8;
			byte=ln*10%8;
			for (i=0; i<64; i++)
			{
				//   (     )
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				if (Color[BG_COLOR]==White)
				{
					LCD_write_data((0x00<<byte)|tmp_data);
				}
				else
				{
					LCD_write_data((0xFF<<byte)|tmp_data);
				}
				if (byte)
				{
					//   (     )
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					// , 2 , ..     
					tmp_data=LCD_read_data(); 
					tmp_data=LCD_read_data();  
					//  
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					if (Color[BG_COLOR]==White)
					{
						LCD_write_data((0x00>>(8-byte))|(tmp_data));
					}
					else
					{
						LCD_write_data((0xFF>>(8-byte))|(tmp_data));
					}					
				}
			}
			page=ln*10/8+1;
			byte=ln*10%8;
			for (i=0; i<64; i++)
			{
				//   (     )
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				if (Color[BG_COLOR]==White)
				{
					LCD_write_data((0x00<<byte)|tmp_data);
				}
				else
				{
					LCD_write_data((0x03<<byte)|tmp_data);
				}
				if (byte)
				{
					//   (     )
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					// , 2 , ..     
					tmp_data=LCD_read_data(); 
					tmp_data=LCD_read_data();  
					//  
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					if (Color[BG_COLOR]==White)
					{
						LCD_write_data((0x00>>(8-byte))|(tmp_data));
					}
					else
					{
						LCD_write_data((0x03>>(8-byte))|(tmp_data));
					}					
				}
			}
			MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
			delay(500);
			MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
			page=ln*10/8;
			byte=ln*10%8;
			for (i=0; i<64; i++)
			{
				//   (     )
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				if (Color[BG_COLOR]==White)
				{
					LCD_write_data((0x00<<byte)|tmp_data);
				}
				else
				{
					LCD_write_data((0xFF<<byte)|tmp_data);
				}
				if (byte)
				{
					//   (     )
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					// , 2 , ..     
					tmp_data=LCD_read_data(); 
					tmp_data=LCD_read_data();  
					//  
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					if (Color[BG_COLOR]==White)
					{
						LCD_write_data((0x00>>(8-byte))|(tmp_data));
					}
					else
					{
						LCD_write_data((0xFF>>(8-byte))|(tmp_data));
					}					
				}
			}
			page=ln*10/8+1;
			byte=ln*10%8;
			for (i=0; i<64; i++)
			{
				//   (     )
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();  
				//  
				LCD_write_cmd(0xB8 | page);																		//Set page
				LCD_write_cmd(0x40 | (i));																		//Set address
				if (Color[BG_COLOR]==White)
				{
					LCD_write_data((0x00<<byte)|tmp_data);
				}
				else
				{
					LCD_write_data((0x03<<byte)|tmp_data);
				}
				if (byte)
				{
					//   (     )
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					// , 2 , ..     
					tmp_data=LCD_read_data(); 
					tmp_data=LCD_read_data();  
					//  
					LCD_write_cmd(0xB8 | page+1);																		//Set page
					LCD_write_cmd(0x40 | (i));																		//Set address
					if (Color[BG_COLOR]==White)
					{
						LCD_write_data((0x00>>(8-byte))|(tmp_data));
					}
					else
					{
						LCD_write_data((0x03>>(8-byte))|(tmp_data));
					}					
				}
			}
			break;			
		}
	}
}

/*******************************************************************************
* Draw bargraph                                                                *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   w:        maximum width of bargraph (in pixels)            *
*                   h:        bargraph height                                  *
*                   val:      value of active bargraph (in 1/100) 	           *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Bargraph (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned int val) 
{
  int i,j;
	unsigned short temp_color=0;
	
	for (i=x; i<(x+w); i++)
	{
		GLCD_PutPixel(i, 		y);
		GLCD_PutPixel(i, 	y+h);
	}
	for (i=y; i<(y+h); i++)
	{
		GLCD_PutPixel(x, 		i);
		GLCD_PutPixel(x+w, 	i);
	}
	GLCD_PutPixel(x+w, y+h);
	for (i=x+1; i<=x+w-1; i++)
	{
		if (i>(x+(w*val/100)))
		{
			temp_color=Color[TXT_COLOR];
			Color[TXT_COLOR]=Color[BG_COLOR];
		}
		for (j=y+1; j<=y+h-1; j++)
		{
			GLCD_PutPixel(i, j);
		}
	}
	Color[TXT_COLOR]=temp_color;
}

/*******************************************************************************
* Display graphical bitmap image at position x horizontally and y vertically   *
* (This function is optimized for 16 bits per pixel format, it has to be       *
*  adapted for any other bits per pixel format)                                *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   w:        width of bitmap                                  *
*                   h:        height of bitmap                                 *
*                   bitmap:   address at which the bitmap data resides         *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Bitmap (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned char *bitmap) {
  unsigned char i, j;
	unsigned char lim_X, lim_Y;
	//unsigned char

	if (w>128)
	{
		lim_X=128;
	}
	else
	{
		lim_X=w;
	}
	if (h>64)
	{
		lim_Y=64;
	}
	else
	{
		lim_Y=h;
	}
	for (i=0; i<lim_Y; i++)
	{
		for (j=0; j<lim_X; j++)
		{
			if (bitmap[i*w+j]>0x7F)
			//if (bitmap[i*w+j]==0x03)
			{
				Color[TXT_COLOR]=White;
			}
			else
			{
				Color[TXT_COLOR]=Black;
			}
			GLCD_PutPixel(x+j, y+i);
		}
	}
}

/*******************************************************************************
* Scroll content of the whole display for dy pixels vertically                 *
*   Parameter:      dy:       number of pixels for vertical scroll             *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_ScrollVertical (unsigned int dy) 
{
	unsigned char page=0;
	unsigned char byte=0;
	unsigned char i=0, j=0;
	unsigned char buffer[64];
	unsigned char tmp_data=0;
	
	page=dy/8;
	byte=dy%8;
	MDR_PORTB->RXTX					&=~E2;														//Unselect second chip in LCD
	delay(500);
	MDR_PORTB->RXTX					|=E1;															//Select first chip in LCD
	LCD_write_cmd(0xC0);																			//Set start line
	LCD_write_cmd(0xB8 | (7-page));
	for (i=0; i<64; i++)
	{
		LCD_write_cmd(0x40 | (i));
		// , 2 , ..     
		tmp_data=LCD_read_data(); 
		tmp_data=LCD_read_data();
		buffer[i]=tmp_data;
	}
	LCD_write_cmd(0xBF);																			//    
	LCD_write_cmd(0x40);
	for (i=0; i<64; i++)
	{
		LCD_write_data(((buffer[i]<<byte)&0xFF));
	}
	for (j=page+1; j<8; j++)
	{
		LCD_write_cmd(0xC0);
		LCD_write_cmd(0xB8 | (7-j));
		for (i=0; i<64; i++)
		{
			LCD_write_cmd(0x40 | (i));
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();
			buffer[i]=tmp_data;			
		}
		LCD_write_cmd(0xB8 | (7-j+page));
		LCD_write_cmd(0x40);
		for (i=0; i<64; i++)
		{
			LCD_write_data(((buffer[i]<<byte)&0xFF));
		}
		if (byte)
		{
			LCD_write_cmd(0xB8 | (7-j+page+1));
			for (i=0; i<64; i++)
			{
				LCD_write_cmd(0x40 | (i));
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();
				LCD_write_cmd(0x40 | (i));
				LCD_write_data((((buffer[i]>>byte)&0xFF)|tmp_data));
			}//*/
		}
	}
	for (j=0; j<page; j++)
	{
		LCD_write_cmd(0xB8 | (j));
		for (i=0; i<64; i++)
		{
			if (Color[BG_COLOR]==White)
			{
				LCD_write_data(0x00);
			}
			else
			{
				LCD_write_data(0xFF);
			}
		}
	}
	LCD_write_cmd(0xB8 | (page));
	{
		for (i=0; i<64; i++)
		{
			LCD_write_cmd(0x40 | (i));
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();
			LCD_write_cmd(0x40 | (i));
			if (Color[BG_COLOR]==White)
			{
				LCD_write_data((0x00>>byte|tmp_data));
			}
			else
			{
				LCD_write_data((0xFF>>byte|tmp_data));
			}
		}
	}
	MDR_PORTB->RXTX					&=~E1;														//Unselect first chip in LCD
	delay(500);
	MDR_PORTB->RXTX					|=E2;															//Select second chip in LCD
	LCD_write_cmd(0xC0);																			//Set start line
	LCD_write_cmd(0xB8 | (7-page));
	for (i=0; i<64; i++)
	{
		LCD_write_cmd(0x40 | (i));
		// , 2 , ..     
		tmp_data=LCD_read_data(); 
		tmp_data=LCD_read_data();
		buffer[i]=tmp_data;
	}
	LCD_write_cmd(0xBF);																			//    
	LCD_write_cmd(0x40);
	for (i=0; i<64; i++)
	{
		LCD_write_data(((buffer[i]<<byte)&0xFF));
	}
	for (j=page+1; j<8; j++)
	{
		LCD_write_cmd(0xC0);
		LCD_write_cmd(0xB8 | (7-j));
		for (i=0; i<64; i++)
		{
			LCD_write_cmd(0x40 | (i));
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();
			buffer[i]=tmp_data;			
		}
		LCD_write_cmd(0xB8 | (7-j+page));
		LCD_write_cmd(0x40);
		for (i=0; i<64; i++)
		{
			LCD_write_data(((buffer[i]<<byte)&0xFF));
		}
		if (byte)
		{
			LCD_write_cmd(0xB8 | (7-j+page+1));
			for (i=0; i<64; i++)
			{
				LCD_write_cmd(0x40 | (i));
				// , 2 , ..     
				tmp_data=LCD_read_data(); 
				tmp_data=LCD_read_data();
				LCD_write_cmd(0x40 | (i));
				LCD_write_data((((buffer[i]>>byte)&0xFF)|tmp_data));
			}//*/
		}
	}
	for (j=0; j<page; j++)
	{
		LCD_write_cmd(0xB8 | (j));
		for (i=0; i<64; i++)
		{
			if (Color[BG_COLOR]==White)
			{
				LCD_write_data(0x00);
			}
			else
			{
				LCD_write_data(0xFF);
			}
		}
	}
	LCD_write_cmd(0xB8 | (page));
	{
		for (i=0; i<64; i++)
		{
			LCD_write_cmd(0x40 | (i));
			// , 2 , ..     
			tmp_data=LCD_read_data(); 
			tmp_data=LCD_read_data();
			LCD_write_cmd(0x40 | (i));
			if (Color[BG_COLOR]==White)
			{
				LCD_write_data((0x00>>byte|tmp_data));
			}
			else
			{
				LCD_write_data((0xFF>>byte|tmp_data));
			}
		}
	}
}


/*******************************************************************************
* Write a command to the LCD controller                                        *
*   Parameter:      cmd:      command to write to the LCD                      *
*   Return:                                                                    *
*******************************************************************************/
void GLCD_WrCmd (unsigned char cmd) 
{
  LCD_write_cmd(cmd);
}
